package com.nokia.financeapi.config.web;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.TypeFactory;
import com.nokia.financeapi.common.R;
import com.nokia.financeapi.pojo.po.common.RequestLogPo;
import com.nokia.financeapi.service.common.RequestLogService;
import com.nokia.financeapi.utils.AESUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.lang.Nullable;
import org.springframework.util.StopWatch;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * 请求日志拦截器
 */
@Slf4j
@ControllerAdvice
@RequiredArgsConstructor
public class RequestLogHandlerInterceptor implements HandlerInterceptor, RequestBodyAdvice, ResponseBodyAdvice<Object> {
    /**
     * 计时器线程变量
     */
    private static final ThreadLocal<StopWatch> STOP_WATCH_THREAD_LOCAL = new ThreadLocal<>();
    private static final ThreadLocal<RequestLogPo> REQUEST_LOG_PO_THREAD_LOCAL = new ThreadLocal<>();
    private final RequestLogService requestLogService;
    private final ObjectMapper objectMapper;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        // 计时器放入线程变量
        STOP_WATCH_THREAD_LOCAL.set(stopWatch);
        // 日志添加跟踪id
        MDC.put("traceId", UUID.randomUUID().toString().replace("-", ""));
        LocalDateTime now = LocalDateTime.now();
        log.info("请求地址: {} {}", request.getRequestURL().toString(), request.getMethod());
        // 请求头参数
        Map<String, String> headers = new HashMap<>();
        Enumeration<String> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String k	= headerNames.nextElement();
            String v = request.getHeader(k);
            headers.put(k, v);
        }
        log.info("请求头参数: {}", new ObjectMapper().writeValueAsString(headers));
        // 查询参数
        Map<String, String> parameters = new HashMap<>();
        Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            String k	= parameterNames.nextElement();
            String v = request.getParameter(k);
            parameters.put(k, v);
        }
        log.info("查询参数: {}", new ObjectMapper().writeValueAsString(parameters));
        String token = request.getHeader("token");
        if (!StringUtils.hasText(token) || "undefined".equals(token)) {
            token = parameters.get("token");
        }
        log.info("token: {}", token);
        if (!StringUtils.hasText(token)) {
            return forbidden(response);
        }
        try {
            String urlDecodeToken = URLDecoder.decode(token, StandardCharsets.UTF_8);
            log.info("urlDecodeToken: {}", urlDecodeToken);
            String decryptToken = AESUtil.decrypt(urlDecodeToken);
            log.info("decryptToken: {}", decryptToken);
            Map<String, String> map = objectMapper.readValue(decryptToken,
                    TypeFactory.defaultInstance().constructMapType(Map.class, String.class, String.class));
            if (!"FINANCE".equals(map.get("APP_ID"))
                    || !StringUtils.hasText(map.get("LOGIN_ID"))
                    || !StringUtils.hasText(map.get("TIME_STAMP"))
                    || !StringUtils.hasText(map.get("EXPIRE_TIME"))
            ) {
                return forbidden(response);
            }
            LocalDateTime timeStamp = LocalDateTime.parse(map.get("TIME_STAMP"),
                    DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            int expireTime = Integer.parseInt(map.get("EXPIRE_TIME"));
            LocalDateTime expireTimeStamp = timeStamp.plusSeconds(expireTime);
            // token过期
            if (now.isAfter(expireTimeStamp)) {
                return unauthorized(response);
            }
            Map<String, String> requestParameters = new HashMap<>();
            requestParameters.put("request_parameters", objectMapper.writeValueAsString(parameters));
            RequestLogPo requestLogPo = new RequestLogPo();
            REQUEST_LOG_PO_THREAD_LOCAL.set(requestLogPo);
            requestLogPo.setRequestTime(now);
            requestLogPo.setLoginId(map.get("LOGIN_ID"));
            requestLogPo.setPageUrl(map.get("REQUEST_URL"));
            requestLogPo.setApi(request.getRequestURI());
            requestLogPo.setRequestParameters(objectMapper.writeValueAsString(requestParameters));
            requestLogPo.setHeaders(objectMapper.writeValueAsString(headers));
            requestLogPo.setAppId(map.get("APP_ID"));
            requestLogPo.setToken(urlDecodeToken);
            requestLogPo.setTimeStamp(timeStamp);
            requestLogPo.setExpireTime(expireTime);
            requestLogPo.setExpireTimeStamp(expireTimeStamp);
        } catch (Exception e) {
            log.warn("token解密失败: {}", token);
            return forbidden(response);
        }
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler,
                                @Nullable Exception ex) throws Exception {
        log.info("响应状态: {}", response.getStatus());
        StopWatch stopWatch = STOP_WATCH_THREAD_LOCAL.get();
        stopWatch.stop();
        log.info("耗时: {} ms", stopWatch.getTotalTimeMillis());
        STOP_WATCH_THREAD_LOCAL.remove();
        RequestLogPo requestLogPo = REQUEST_LOG_PO_THREAD_LOCAL.get();
        if (requestLogPo != null) {
            requestLogService.offer(requestLogPo);
        }
        REQUEST_LOG_PO_THREAD_LOCAL.remove();
    }

    private boolean forbidden(HttpServletResponse response) throws IOException {
        response.setCharacterEncoding(StandardCharsets.UTF_8.name());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        String r = objectMapper.writeValueAsString(R.error().code(403).message("没有权限访问"));
        byte[] bytes = r.getBytes();
        response.getOutputStream().write(bytes);
        StopWatch stopWatch = STOP_WATCH_THREAD_LOCAL.get();
        stopWatch.stop();
        log.warn("响应状态: {}", response.getStatus());
        log.warn("响应内容: {}", r);
        log.info("耗时 {} ms", stopWatch.getTotalTimeMillis());
        STOP_WATCH_THREAD_LOCAL.remove();
        return false;
    }

    private boolean unauthorized(HttpServletResponse response) throws IOException {
        response.setCharacterEncoding(StandardCharsets.UTF_8.name());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        String r = objectMapper.writeValueAsString(R.error().code(401).message("请刷新页面或重新登录"));
        byte[] bytes = r.getBytes();
        response.getOutputStream().write(bytes);
        StopWatch stopWatch = STOP_WATCH_THREAD_LOCAL.get();
        stopWatch.stop();
        log.warn("响应状态: {}", response.getStatus());
        log.warn("响应内容: {}", r);
        log.info("耗时 {} ms", stopWatch.getTotalTimeMillis());
        STOP_WATCH_THREAD_LOCAL.remove();
        return false;
    }

    @Override
    public boolean supports(MethodParameter methodParameter, Type targetType, Class<? extends HttpMessageConverter<?>> converterType) {
        return true;
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage inputMessage, MethodParameter parameter, Type targetType, Class<? extends HttpMessageConverter<?>> converterType) throws IOException {
        return inputMessage;
    }

    @Override
    public Object afterBodyRead(Object body, HttpInputMessage inputMessage, MethodParameter parameter, Type targetType, Class<? extends HttpMessageConverter<?>> converterType) {
        try {
            if (STOP_WATCH_THREAD_LOCAL.get() != null) {
                String bodyStr = objectMapper.writeValueAsString(body);
                log.info("请求体参数: {}", bodyStr);
                RequestLogPo requestLogPo = REQUEST_LOG_PO_THREAD_LOCAL.get();
                if (requestLogPo != null) {
                    Map<String, Object> requestParameters = objectMapper.readValue(requestLogPo.getRequestParameters(),
                            TypeFactory.defaultInstance().constructMapType(Map.class, String.class, String.class));
                    requestParameters.put("request_body", body);
                    requestLogPo.setRequestParameters(objectMapper.writeValueAsString(requestParameters));
                }
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
        }
        return body;
    }

    @Override
    public Object handleEmptyBody(Object body, HttpInputMessage inputMessage, MethodParameter parameter, Type targetType, Class<? extends HttpMessageConverter<?>> converterType) {
        return body;
    }

    @Override
    public boolean supports(MethodParameter returnType, Class<? extends HttpMessageConverter<?>> converterType) {
        return true;
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter returnType, MediaType selectedContentType, Class<? extends HttpMessageConverter<?>> selectedConverterType, ServerHttpRequest request, ServerHttpResponse response) {
        try {
            if (STOP_WATCH_THREAD_LOCAL.get() != null) {
                log.info("响应内容: {}", org.apache.commons.lang3.StringUtils.substring(objectMapper.writeValueAsString(body), 0, 500));
            }
        } catch (Exception e) {
            log.error(e.toString(), e);
        }
        return body;
    }
}